"""sdk functions"""


def change_led():
    """function for change LED """
    response = "lead change"
    print(response)
    return response


def busser_beep():
    """function for Busser beep"""
    response = "busser beep"
    print(response)
    return response


def get_media():
    """function for get media"""
    response = "get media"
    print(response)
    return response
